package BIDM::Webpage;

=head1 NAME

BIDM::Webpage

=head1 DESCRIPTION

=cut

use strict;

use HTML::Parser;
use HTTP::Request;
use BIDM::Tagset;
use BIDM::Tag;
use BIDM::MemberMixin;

use FileHandle;

use vars qw(@ISA);

@ISA = qw(BIDM::MemberMixin HTML::Parser);

sub new
{
    my($class, $url) = @_;
    my($self);

    $self = {
	_buf => '',
	url => $url,
    };

    bless($self, $class);
    return $self;
}

sub retrieve_metadata
{
    my($self, $user_agent, $last_retrieval) = @_;
    my($req, $res, $url);

    $self->{page} = new FileHandle(">/tmp/page.out");

    $url = $self->url();
#    print "Fetching url $url\n";

    $req = new HTTP::Request("GET", $url);

    if (defined($last_retrieval) && $last_retrieval > 0)
    {
#	print "setting if-modified to $last_retrieval\n";
	$req->if_modified_since($last_retrieval);
    }

    $self->{bidm_tagset} = new BIDM::Tagset();

    $res = $user_agent->request($req,
			       sub { $self->chunk(@_); });

    #
    # Save the headers for future reference.
    #
    $self->response($res);
    $res->date(time);
    $self->{bidm_tagset}->response($res);
    
    if ($res->is_success)
    {
#	print "Success! last-mod=", $res->last_modified(), "\n";
	$self->parse(undef);
	if ($self->{page})
	{
	    $self->{page}->close();
	    delete $self->{page};
	}
	return ($self->{bidm_tagset}, $res);
    }
    else
    {
	if ($self->{page})
	{
	    $self->{page}->close();
	    delete $self->{page};
	}
#	print "Cannot retrieve $url: ", $res->code(), "\n";
	return (undef, $res);
    }
}

sub chunk
{
    my($self, $content, $response, $protocol) = @_;
    my($fh);
    if ($fh = $self->{page})
    {
	print $fh $content;
    }
    $self->parse($content);
}

sub start
{
    my($self, $tag, $attr) = @_;
    my($bidm_tag);

    
    if ($tag eq 'meta' or $tag eq 'link')
    {
#	print "start '$tag'\n";
#	my($k, $v);
#	while (($k, $v) = each(%$attr))
#	{
#	    print "$k: $v\n";
#	}
	$bidm_tag = eval { BIDM::Tag::create_from_tag($tag, $attr);};
	die "create failed: $@" if $@;
#	print "Created tag $bidm_tag\n";
#	$bidm_tag->print();
	$self->{bidm_tagset}->add_tag($bidm_tag);
    }
}

sub url { shift->_elem('url', @_); }
sub response { shift->_elem('response', @_); }

1;
